@@ -4,15 +4,20 @@ import java.text.ParseException;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
+import java.util.Map;
+import java.util.Set;
 import java.util.concurrent.CopyOnWriteArrayList;
 
 import org.apache.log4j.Logger;
+import org.codehaus.jackson.Version;
+import org.codehaus.jackson.util.VersionUtil;
 import org.codehaus.jettison.json.JSONArray;
 import org.codehaus.jettison.json.JSONException;
 import org.codehaus.jettison.json.JSONObject;
 
 import eu.emi.emir.EMIRServer;
 import eu.emi.emir.ServerProperties;
+import eu.emi.emir.client.FacetKeyType;
 import eu.emi.emir.client.ServiceBasicAttributeNames;
 import eu.emi.emir.client.util.Log;
 import eu.emi.emir.db.ExistingResourceException;
@@ -33,7 +38,7 @@ import com.mongodb.MongoException.DuplicateKey;
 import com.mongodb.util.JSON;
 
 /**
- * Mongodb integration class to proxy database operations 
+ * Mongodb integration class to proxy database operations
  * 
  * @author martoni
  * @author a.memon
@@ -43,77 +48,114 @@ import com.mongodb.util.JSON;
 public class MongoDBServiceDatabase implements ServiceDatabase {
 	private static Logger logger = Log.getLogger(Log.EMIR_DB,
 			MongoDBServiceDatabase.class);
-	private static Mongo connection;
+	private static MongoClient connection;
 	private DB database;
 	private DBCollection serviceCollection;
-
+	private String hostName;
+	private Integer port;
+	private String colName;
+	private String dbName;
+	private String userName;
+	private String password;
+	private static String dbVersion;
+	
+	
+	
 	public MongoDBServiceDatabase() {
 		if (EMIRServer.getServerProperties() == null) {
 			new EMIRServer();
 		}
-		String hostname = EMIRServer.getServerProperties().getValue(ServerProperties.PROP_MONGODB_HOSTNAME);
-		Integer port = EMIRServer.getServerProperties().getIntValue(ServerProperties.PROP_MONGODB_PORT);
-		
-		String dbName = EMIRServer.getServerProperties().getValue(ServerProperties.PROP_MONGODB_DB_NAME);
-		String colName = EMIRServer.getServerProperties().getValue(ServerProperties.PROP_MONGODB_COLLECTION_NAME);
+		hostName = EMIRServer.getServerProperties().getValue(
+				ServerProperties.PROP_MONGODB_HOSTNAME);
+		port = EMIRServer.getServerProperties().getIntValue(
+				ServerProperties.PROP_MONGODB_PORT);
 
-		String dbUserName = EMIRServer.getServerProperties().getValue(ServerProperties.PROP_MONGODB_USERNAME);
-		String dbPassword = EMIRServer.getServerProperties().getValue(ServerProperties.PROP_MONGODB_PASSWORD);
+		dbName = EMIRServer.getServerProperties().getValue(
+				ServerProperties.PROP_MONGODB_DB_NAME);
+		colName = EMIRServer.getServerProperties().getValue(
+				ServerProperties.PROP_MONGODB_COLLECTION_NAME);
+
+		userName = EMIRServer.getServerProperties().getValue(
+				ServerProperties.PROP_MONGODB_USERNAME);
+		password = EMIRServer.getServerProperties().getValue(
+				ServerProperties.PROP_MONGODB_PASSWORD);
 
 		try {
 			if (connection == null) {
 
-				MongoOptions mo = new MongoOptions();
-				mo.autoConnectRetry = true;
-				mo.connectTimeout = 100;
-				mo.maxWaitTime = 100;
-				mo.socketKeepAlive = true;
-				mo.connectionsPerHost = 255;
-
-				ServerAddress sa = new ServerAddress(hostname,
+				MongoClientOptions mo = MongoClientOptions.builder().autoConnectRetry(true).connectTimeout(5000).maxAutoConnectRetryTime(10000).maxWaitTime(5000).socketKeepAlive(true).socketTimeout(0).connectionsPerHost(255).build();
+				
+				ServerAddress sa = new ServerAddress(hostName,
 						Integer.valueOf(port));
-				connection = new Mongo(sa, mo);
+				connection = new MongoClient(sa, mo);
 			}
 
 			database = connection.getDB(dbName);
 
-			if (dbUserName != null) {
-				if ((!dbUserName.equalsIgnoreCase(""))
-						&& (!dbPassword.equalsIgnoreCase(""))) {
-					if (!database
-							.authenticate(dbUserName, dbPassword.toCharArray())) {
-						
-						Log.logException("Cannot authenticate the user: " + dbUserName + "\nProvide the correct MongoDB database username and password in configuration file and restart the EMIR server again", new RuntimeException("MongoDB Authentication Failed"));
-						
+			if (userName != null) {
+				if ((!userName.equalsIgnoreCase(""))
+						&& (!password.equalsIgnoreCase(""))) {
+					if (!database.authenticate(userName,
+							password.toCharArray())) {
+
+						Log.logException(
+								"Cannot authenticate the user: "
+										+ userName
+										+ "\nProvide the correct MongoDB database username and password in configuration file and restart the EMIR server again",
+								new RuntimeException(
+										"MongoDB Authentication Failed"));
+
 						System.out
 								.printf("%s:%s.%s.%s",
 										"Error occurred while starting the EMIR server",
 										"Cannot authenticate the database User: "
-												+ dbUserName,
+												+ userName,
 										" Provide the correct MongoDB database username and password in configuration file and restart the EMIR server again",
 										" Stoppoing the EMIR server.");
 						System.exit(1);
 					}
 
-				}	
+				}
 			}
-			
 
 			serviceCollection = database.getCollection(colName);
 
 			// setting index and unique constraint on "service endpoint id"
-			BasicDBObject obj = new BasicDBObject(
-					ServiceBasicAttributeNames.SERVICE_ENDPOINT_ID
-							.getAttributeName(),
-					"1");
-
-			serviceCollection.ensureIndex(obj,
-					ServiceBasicAttributeNames.SERVICE_ENDPOINT_ID
-							.getAttributeName(), true);
+			if (isVersionNumber241OrGreater()) {
+				BasicDBObject obj = new BasicDBObject(
+						ServiceBasicAttributeNames.SERVICE_ENDPOINT_ID.getAttributeName(),
+								1);
+				serviceCollection.ensureIndex(obj,
+						ServiceBasicAttributeNames.SERVICE_ENDPOINT_ID
+								.getAttributeName(), true);
+				
+				
+	            BasicDBObject nonUniqueIndexKeys = new BasicDBObject();
+				nonUniqueIndexKeys.put(ServiceBasicAttributeNames.SERVICE_NAME.getAttributeName(), 1);
+				nonUniqueIndexKeys.put(ServiceBasicAttributeNames.SERVICE_TYPE.getAttributeName(), 1);
+				nonUniqueIndexKeys.put(ServiceBasicAttributeNames.SERVICE_ENDPOINT_CAPABILITY.getAttributeName(),1);
+				serviceCollection.ensureIndex(nonUniqueIndexKeys, "secondary",false);	
+			} else {
+				//mongodb versions lesser than 2.4.1 initialize indexes using string value
+				BasicDBObject obj = new BasicDBObject(
+						ServiceBasicAttributeNames.SERVICE_ENDPOINT_ID.getAttributeName(),
+								"1");
+				serviceCollection.ensureIndex(obj,
+						ServiceBasicAttributeNames.SERVICE_ENDPOINT_ID
+								.getAttributeName(), true);
+				
+				
+	            BasicDBObject nonUniqueIndexKeys = new BasicDBObject();
+				nonUniqueIndexKeys.put(ServiceBasicAttributeNames.SERVICE_NAME.getAttributeName(), "1");
+				nonUniqueIndexKeys.put(ServiceBasicAttributeNames.SERVICE_TYPE.getAttributeName(), "1");
+				nonUniqueIndexKeys.put(ServiceBasicAttributeNames.SERVICE_ENDPOINT_CAPABILITY.getAttributeName(),"1");
+				serviceCollection.ensureIndex(nonUniqueIndexKeys, "secondary",false);
+			}
+			
 		} catch (MongoException e) {
 			Log.logException("", e, logger);
 			logger.warn(e.getCause());
-		} catch (Exception e) {			
+		} catch (Exception e) {
 			logger.error("Error in connecting the MongoDB database", e);
 		}
 
@@ -132,9 +174,23 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 	public MongoDBServiceDatabase(String hostname, Integer port, String dbName,
 			String colName) {
 		try {
-			
+			this.hostName = hostname;
+			this.dbName = dbName;
+			this.colName = colName;
+			this.port = port;
+			init();
+		} catch (Exception e) {
+			logger.error("Error in connecting the MongoDB database", e);
+
+		}
+	}
+	
+	
+	private void init(){
+		try {
+
 			if (connection == null) {
-				connection = new Mongo(hostname, port);
+				connection = new MongoClient(hostName, port);
 			}
 
 			database = connection.getDB(dbName);
@@ -142,21 +198,14 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 			serviceCollection = database.getCollection(colName);
 
 			// setting index and uniquesness on service url
-			BasicDBObject obj = new BasicDBObject(
-					ServiceBasicAttributeNames.SERVICE_ENDPOINT_ID
-							.getAttributeName(),
-					"1");
 			
-//			obj.put(ServiceBasicAttributeNames.SERVICE_NAME.getAttributeName(),
-//					"1");
-//			
-//			obj.put(ServiceBasicAttributeNames.SERVICE_TYPE.getAttributeName(),"1");
-//					
-//			obj.put(ServiceBasicAttributeNames.SERVICE_ENDPOINT_CAPABILITY.getAttributeName(),"1");
-
-			serviceCollection.ensureIndex(obj,
+			BasicDBObject obj = getUniqueIndexes();
+					
+					
+			serviceCollection.ensureIndex(getUniqueIndexes(),
 					ServiceBasicAttributeNames.SERVICE_ENDPOINT_ID
 							.getAttributeName(), true);
+			serviceCollection.ensureIndex(getNonUniqueIndexes());
 			if (logger.isDebugEnabled()) {
 				logger.debug("Unique index created: " + obj);
 			}
@@ -165,7 +214,23 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 
 		}
 	}
-
+	
+	private BasicDBObject getUniqueIndexes(){
+		BasicDBObject obj = new BasicDBObject(
+				ServiceBasicAttributeNames.SERVICE_ENDPOINT_ID
+						.getAttributeName(),
+				1);
+		return obj;
+	}
+	
+	private BasicDBObject getNonUniqueIndexes(){
+		BasicDBObject nonUniqueIndexKeys = new BasicDBObject();
+		nonUniqueIndexKeys.put(ServiceBasicAttributeNames.SERVICE_NAME.getAttributeName(), 1);
+		nonUniqueIndexKeys.put(ServiceBasicAttributeNames.SERVICE_TYPE.getAttributeName(), 1);
+		nonUniqueIndexKeys.put(ServiceBasicAttributeNames.SERVICE_ENDPOINT_CAPABILITY.getAttributeName(),1);
+		return nonUniqueIndexKeys;
+	}
+	
 	@Override
 	public void insert(ServiceObject item) throws ExistingResourceException,
 			PersistentStoreFailureException {
@@ -180,10 +245,11 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 			// .getAttributeName(), new Date());
 
 			serviceCollection.insert(db, WriteConcern.SAFE);
-			
+
 			database.requestDone();
-			
-			logger.info("inserted Service Endpoint record with ID: " + item.getEndpointID());
+
+			logger.info("inserted Service Endpoint record with ID: "
+					+ item.getEndpointID());
 			// EventDispatcher.notifyRecievers(new Event(EventTypes.SERVICE_ADD,
 			// item
 			// .toJSON()));
@@ -222,7 +288,7 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 
 		return so;
 	}
-	
+
 	@Override
 	public ServiceObject getServiceByEndpointID(String identifier)
 			throws MultipleResourceException, NonExistingResourceException,
@@ -248,8 +314,9 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 	}
 
 	@Override
-	public void deleteByEndpointID(String endpointID) throws MultipleResourceException,
-			NonExistingResourceException, PersistentStoreFailureException {
+	public void deleteByEndpointID(String endpointID)
+			throws MultipleResourceException, NonExistingResourceException,
+			PersistentStoreFailureException {
 		database.requestStart();
 		database.requestEnsureConnection();
 		BasicDBObject query = new BasicDBObject();
@@ -260,14 +327,15 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 		database.requestDone();
 		if (d == null) {
 			if (logger.isDebugEnabled()) {
-				String msg = "No service description with the Endpoint ID:" + endpointID
-						+ " exists";
+				String msg = "No service description with the Endpoint ID:"
+						+ endpointID + " exists";
 				logger.debug(msg);
 			}
 			throw new NonExistingResourceException(
-					"No service description with the Endpoint ID:" + endpointID + " exists");
+					"No service description with the Endpoint ID:" + endpointID
+							+ " exists");
 		}
-		logger.info("deleted: " + endpointID);
+		logger.info("DELETED Service Endpoint Record with ID: " + endpointID);
 		// sending delete event to the receivers
 		try {
 			JSONObject deletedEntry = new JSONObject(d.toString());
@@ -283,7 +351,7 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 	public void update(ServiceObject sObj) throws MultipleResourceException,
 			NonExistingResourceException, PersistentStoreFailureException {
 		try {
-			String  id = sObj.getServiceID();
+			String id = sObj.getEndpointID();
 			logger.debug("updating service description: " + sObj);
 
 			database.requestStart();
@@ -295,8 +363,9 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 			BasicDBObject query = new BasicDBObject();
 			query.put(ServiceBasicAttributeNames.SERVICE_ENDPOINT_ID
 					.getAttributeName(), sObj.getEndpointID());
-			
-			// Check the entry is exist or not. Validation need if it is not exist before.
+
+			// Check the entry is exist or not. Validation need if it is not
+			// exist before.
 			DBObject db = serviceCollection.findOne(new BasicDBObject(
 					ServiceBasicAttributeNames.SERVICE_ENDPOINT_ID
 							.getAttributeName(), sObj.getEndpointID()));
@@ -304,29 +373,40 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 				try {
 					new RegistrationValidator().validateInfo(sObj.toJSON());
 				} catch (ConfigurationException e) {
-					Log.logException("Error during the update message validation. (ID:"+id+")", e, logger);
+					Log.logException(
+							"Error during the update message validation. (ID:"
+									+ id + ")", e, logger);
 					return;
 				} catch (InvalidServiceDescriptionException e) {
-					Log.logException("Error during the update message validation. (ID:"+id+")", e, logger);
+					Log.logException(
+							"Error during the update message validation. (ID:"
+									+ id + ")", e, logger);
 					return;
 				} catch (JSONException e) {
-					Log.logException("Error during the update message validation. (ID:"+id+")", e, logger);
+					Log.logException(
+							"Error during the update message validation. (ID:"
+									+ id + ")", e, logger);
 					return;
 				} catch (ParseException e) {
-					Log.logException("Error during the update message validation. (ID:"+id+")", e, logger);
+					Log.logException(
+							"Error during the update message validation. (ID:"
+									+ id + ")", e, logger);
 					return;
 				}
 			}
 
-			serviceCollection.update(query, dbObj, true, false); //upsert=true and multi=false
+			serviceCollection.update(query, dbObj, true, false); // upsert=true
+																	// and
+																	// multi=false
 			database.requestDone();
-			logger.info("updated Service Endpoint Record with ID: " + id);
+			logger.info("CREATED/UPDATED Service Endpoint Record with ID: " + id);
 			// sending update event to the receivers
 			// EventDispatcher.notifyRecievers(new
 			// Event(EventTypes.SERVICE_UPDATE,
 			// sObj.toJSON()));
 		} catch (MongoException e) {
-			Log.logException("Error updating the Service Record in MongoDB: "+sObj, e, logger);
+			Log.logException("Error updating the Service Record in MongoDB: "
+					+ sObj, e, logger);
 		}
 
 	}
@@ -345,6 +425,7 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 				ServiceObject s = new ServiceObject(cur.next().toString());
 				resultCollection.add(s);
 			}
+			logger.debug(JSON.serialize(cur.explain()));
 			cur.close();
 		} catch (Exception e) {
 			Log.logException("", e);
@@ -374,6 +455,7 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 				ServiceObject s = new ServiceObject(cur.next());
 				resultCollection.add(s);
 			}
+			logger.debug(JSON.serialize(cur.explain()));
 			cur.close();
 		} catch (Exception e) {
 			Log.logException("", e);
@@ -399,6 +481,7 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 				ServiceObject s = new ServiceObject(cur.next().toString());
 				resultCollection.add(s);
 			}
+			logger.debug(JSON.serialize(cur.explain()));
 			cur.close();
 		} catch (Exception e) {
 			Log.logException("", e);
@@ -415,6 +498,7 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 		while (cur.hasNext()) {
 			arr.put(new JSONObject(JSON.serialize(cur.next())));
 		}
+		logger.debug(JSON.serialize(cur.explain()));
 		cur.close();
 		return arr;
 	}
@@ -440,6 +524,7 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 			while (cur.hasNext()) {
 				arr.put(new JSONObject(JSON.serialize(cur.next())));
 			}
+			logger.debug(JSON.serialize(cur.explain()));
 			cur.close();
 		} catch (Exception e) {
 			Log.logException("", e);
@@ -462,6 +547,7 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 			while (cur.hasNext()) {
 				arr.put(new JSONObject(JSON.serialize(cur.next())));
 			}
+			logger.debug(JSON.serialize(cur.explain()));
 			cur.close();
 		} catch (Exception e) {
 			Log.logException("", e);
@@ -488,6 +574,7 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 			while (cur.hasNext()) {
 				arr.put(new JSONObject(JSON.serialize(cur.next())));
 			}
+			logger.debug(JSON.serialize(cur.explain()));
 			cur.close();
 		} catch (Exception e) {
 			Log.logException("", e);
@@ -518,7 +605,8 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 	}
 
 	@Override
-	public JSONArray paginatedQuery(String query, Integer pageSize, String id, String orderBy) {
+	public JSONArray paginatedQuery(String query, Integer pageSize, String id,
+			String orderBy) {
 		DBObject queryObj = (DBObject) JSON.parse(query);
 		DBCursor cur = null;
 		if (orderBy == null || orderBy.isEmpty()) {
@@ -541,7 +629,7 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 				db.putAll(queryObj);
 			}
 			cur = serviceCollection.find(db).sort(OrderBy).limit(pageSize);
-
+			logger.debug(JSON.serialize(cur.explain()));
 		}
 
 		JSONArray arr = new JSONArray();
@@ -549,6 +637,7 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 			while (cur.hasNext()) {
 				arr.put(new JSONObject(JSON.serialize(cur.next())));
 			}
+			logger.debug(JSON.serialize(cur.explain()));
 			cur.close();
 		} catch (Exception e) {
 			Log.logException("", e);
@@ -604,7 +693,7 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 		database.requestStart();
 		database.requestEnsureConnection();
 		serviceCollection.remove(db);
-		
+
 		database.requestDone();
 
 	}
@@ -619,6 +708,15 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 	public void dropDB() {
 		database.dropDatabase();
 	}
+	
+	public void dropDB(String dbName) {
+		connection.dropDatabase(dbName);
+	}
+	
+	public DB createOrGetDB(String name){
+		
+		return connection.getDB(name);
+	}
 
 	/*
 	 * (non-Javadoc)
@@ -627,7 +725,173 @@ public class MongoDBServiceDatabase implements ServiceDatabase {
 	 */
 	@Override
 	public String getDBVersion() {
-		CommandResult result = database.command("serverStatus");
-		return result.getString("version");
+		if (dbVersion == null) {
+			CommandResult result = database.command("serverStatus");
+			dbVersion = result.getString("version");
+		}
+		
+		return dbVersion;
 	}
+
+	/*
+	 * (non-Javadoc)
+	 * 
+	 * @see eu.emi.emir.db.ServiceDatabase#size()
+	 */
+	@Override
+	public Long size() {
+		if (serviceCollection == null) {
+			return 0L;
+		}
+		Long size = serviceCollection.count();
+		return size;
+	}
+
+	/*
+	 * (non-Javadoc)
+	 * 
+	 * @see eu.emi.emir.db.ServiceDatabase#facetedQuery()
+	 */
+	@Deprecated
+	public JSONArray facetedQuery(Set<String> j) throws JSONException {
+		JSONArray ja = new JSONArray();
+
+		for (String str : j) {
+
+			// create match
+			BasicDBObject match = new BasicDBObject();
+			match.put("$match", new BasicDBObject());
+
+			// build the $projection operation
+			DBObject fields = new BasicDBObject();
+
+			// fields.put("count", 1);
+			DBObject project = new BasicDBObject();
+
+			// Now the $group operation
+			DBObject groupFields = new BasicDBObject();
+
+			// build the $projection operation
+			fields.put(str, 1);
+			project.put("$project", fields);
+
+			// Now the $group operation
+			groupFields.put("_id", "$" + str);
+
+			// performing sum and storing it in the count attribute
+			groupFields.put("count", new BasicDBObject("$sum", 1));
+
+			DBObject group = new BasicDBObject("$group", groupFields);
+			AggregationOutput output = serviceCollection.aggregate(match,
+					project, group);
+
+			Iterable<DBObject> it = output.results();
+
+			JSONArray terms = new JSONArray();
+
+			for (DBObject dbObject : it) {
+				JSONObject term = new JSONObject(JSON.serialize(dbObject));
+				System.out.println(term);
+				terms.put(term);
+			}
+
+			System.out.println("\n\n");
+
+			JSONObject attrName = new JSONObject();
+
+			attrName.put(str, terms);
+			ja.put(attrName);
+		}
+
+		return ja;
+	}
+
+	/*
+	 * (non-Javadoc)
+	 * 
+	 * @see eu.emi.emir.db.ServiceDatabase#facetedQuery(java.util.Map)
+	 */
+	@Override
+	public JSONArray facetedQuery(Map<String, String> map) throws Exception {
+		if (!isVersionNumber241OrGreater()) {
+			UnsupportedOperationException u = new UnsupportedOperationException("The MongoDB server version should be equal to or greater than: 2.4.1, current version is: "+getDBVersion());
+			Log.logException("Faceted query is not supported in the current backend MongoDB, please update to the version: "+getDBVersion(), u);
+			throw u;
+		}
+		
+		JSONArray ja = new JSONArray();
+		Set<String> j = map.keySet();
+		for (String key : j) {
+
+			// create match
+			BasicDBObject match = new BasicDBObject();
+			match.put("$match", new BasicDBObject());
+
+			// build the $projection operation
+			DBObject fields = new BasicDBObject();
+
+			// fields.put("count", 1);
+			DBObject project = new BasicDBObject();
+
+			// Now the $group operation
+			DBObject groupFields = new BasicDBObject();
+
+			DBObject unwindFields = new BasicDBObject();
+
+			
+
+			// build the $projection operation
+			fields.put(key, 1);
+			project.put("$project", fields);
+
+			// Now the $group operation
+			groupFields.put("_id", "$" + key);
+
+			// performing sum and storing it in the count attribute
+			groupFields.put("count", new BasicDBObject("$sum", 1));
+
+			DBObject group = new BasicDBObject("$group", groupFields);
+			
+			AggregationOutput output = null;
+			
+			if (map.get(key).equalsIgnoreCase(FacetKeyType.ARRAY)) {
+				unwindFields
+				.put("$unwind",
+						"$"
+								+ key);
+				
+				output = serviceCollection.aggregate(match,
+						unwindFields, project, group);	
+			} else if (map.get(key).equalsIgnoreCase(FacetKeyType.SIMPLE)) {
+				output = serviceCollection.aggregate(match, project, group);
+			}
+			
+
+			Iterable<DBObject> it = output.results();
+
+			JSONArray terms = new JSONArray();
+			for (DBObject dbObject : it) {
+				JSONObject term = new JSONObject(JSON.serialize(dbObject));
+				terms.put(term);
+			}
+
+			JSONObject attrName = new JSONObject();
+
+			attrName.put(key, terms);
+			ja.put(attrName);
+		}
+
+		return ja;
+	}
+
+	
+	public boolean isVersionNumber241OrGreater(){
+		Version v =new Version(2, 4, 1, null);
+		if (VersionUtil.parseVersion(getDBVersion()).compareTo(v) < 0) {
+			return false; 
+		};
+		return true;
+	}
+	
+
 }
\ No newline at end of file
